//---------------------------------------------------
// File name   : snmakedeps.e
// Project     : AEWare
// Developers  : Steve Hobbs, Cadence Design Systems
// Created     : Thu Nov 27 18:53:27 GMT 2008
// Description : Specman Makefile dependency generator
// Notes       : Load this or compile it into Specman,
//             : then use the command:
//             : "write makefile <filename> [-ignore_test]"
// --------------------------------------------------
// Copyright 2009 (c) Cadence Design Systems
// --------------------------------------------------
<'
struct dependencies {
  !name : string;
  !filelist : list of string;
};

extend sys {
  snmakedeps(filename : string, ignore_test_files : bool) is {
    var current_target : dependencies;
    var deps       : list(key:name) of dependencies;
    var compmods   : list of rf_module;
    var top_module : rf_module;
    var makefile   : file = files.open(filename, "w", "Makefile");
    out("Generating Makefile dependencies for loaded a modules.");

    files.write(makefile, "## Auto-generated Makefile for Specman - DO NOT EDIT");

    for each (m) in rf_manager.get_user_modules() {
      var modulename : string = m.get_name();
      top_module = m;
      if ( (modulename ~ "snmakedeps") ) || ( ignore_test_files && (modulename ~ "/^test/") ) {
        outf("Ignoring file %s\n", modulename);
      } else {
        var filename    : string = m.get_full_file_name();
        --var pkgname  : string = m.get_package().get_name();
        var module_name : string = m.get_name();
        if (NULL == current_target) {
          current_target = new;
          current_target.filelist = {filename};
        } else {
          current_target.filelist.add(filename);
        };
        --if ( m.is_compiled() && m.is_top_module() ) {
        if ( m.is_top_module() ) {
          compmods.add(m);
          current_target.name = module_name;
          deps.add(current_target);
          current_target = NULL;
        };
      };
    };
    for each (m) in compmods {
      var m_name : string = m.get_name();
      files.write(makefile, appendf("## compiled module %s\n%s_files=\\", m_name, m_name));
      for i from 0 to deps.key(m_name).filelist.size()-2 do {
        files.write(makefile, appendf("\t%s \\", deps.key(m_name).filelist[i]));
      };
      files.write(makefile, appendf("\t%s\n", deps.key(m_name).filelist[deps.key(m_name).filelist.size()-1]));  // last item has no trailing backslash!
    };

    // Generate sn_compile targets for each compiled chunk
    var i : uint = 0;
    for each (m) in compmods {
      // First, the basic dependency on the files loaded by this module:
      var tgt : string = appendf("libsn_%s.so: $(%s_files)", m.get_name(), m.get_name() );
      // Next, if this is not the only top-module, depend on the previous top modules:
      if (i>0) {
        for j from 0 to i-1 do {
          tgt = appendf("%s libsn_%s.so", tgt, compmods[j].get_name() );
        };
      };
      files.write(makefile, tgt);
      files.write(makefile, appendf("\tsn_compile.sh -exe -shlib -enable_DAC %s\n", m.get_full_file_name()));
      i += 1;
    };
    files.write(makefile, appendf("sncompile: libsn_%s.so\n\techo 'Recompiling e code...'\n\n", compmods[compmods.size()-1].get_name()));

    // Build a Make target for generating any chosen stub file
    // Chooses to run the compiled exe if it exists, else loads the e code
    files.write(makefile, appendf("# Use 'make stub_<sn_adapter_name>' to build the desired stubs file,"));
    files.write(makefile, appendf("# for example: 'make stub_ncvhdl'."));

    var tgt : string = appendf("stub_%%: ");
    for each (m) in compmods {
      tgt = appendf("%s $(%s_files)", tgt, m.get_name() );
    };
    files.write(makefile, tgt);
    if ( compmods[compmods.size()-1].get_name() == top_module.get_name() ) {
      // Is the compiled top the same as the final module that was loaded
      // (i.e. test file)
      files.write(makefile, appendf("\tif [ -x ./%s ]; then\\\n\t ./%s -c 'write stubs -$*'; \\",
                                    compmods[compmods.size()-1].get_name(),
                                    compmods[compmods.size()-1].get_name() ));

    } else {
      files.write(makefile, appendf("\tif [ -x ./%s ]; then\\\n\t ./%s -c 'load %s ; write stubs -$*'; \\",
                                    compmods[compmods.size()-1].get_name(),
                                    compmods[compmods.size()-1].get_name(),
                                    top_module.get_full_file_name() ));
    };
    files.write(makefile, appendf("\telse\\\n\tspecman -c 'load %s ; write stubs -$*';\\\n\tfi\n",
                                  top_module.get_full_file_name()));

    // Special target to create a tarball of all Specman files
    tgt = appendf("tarball: ");
    var cmd : string = appendf("\ttar -zcvhf %s.tgz", top_module.get_name());
    for each (m) in compmods {
      tgt = appendf("%s $(%s_files)", tgt, m.get_name() );
      cmd = appendf("%s $(%s_files)", cmd, m.get_name() );
    };
    files.write(makefile, tgt);
    cmd = appendf("%s\n\techo 'Created tarball of the Specman code. Filename is %s.tgz'\n", cmd, top_module.get_name() );
    files.write(makefile, cmd);

    files.close(makefile);
  };
};

define <write_makefile_dependencies'command> "write makefile <filename'file>[ <ignore_test'exp>]" as computed {
  if (<ignore_test'exp> == "-ignore_test") || (<ignore_test'exp> == "-ignore") {
    return appendf("sys.snmakedeps(\"%s\",TRUE)",<filename'file>);
  } else {
    return appendf("sys.snmakedeps(\"%s\",FALSE)",<filename'file>);
  };
};
'>

